import { useMemo, forwardRef, createElement } from 'react';

function parseDefinition(propValue, propDefinition, prefix = "") {
    if (typeof propDefinition === 'string') { // prop: string
        if (propValue === true && propDefinition) {
            return propDefinition.split(/\s+/).map(s => prefix + s.trim()); // split string into multiple parts because we always return array of single classes
        }
    }
    else if (propDefinition instanceof Array) { //  prop: string[]
        if (propValue === true) {
            return propDefinition.flatMap(d => parseDefinition(true, d, prefix));
        }
    }
    else if (typeof propDefinition === 'function') {
        return parseDefinition(true, propDefinition(propValue), prefix);
    }
    else if (typeof propDefinition === 'object') { //  prop: {key: value?}  // propDefinition.toString() === "[object Object]" 
        // test if propValue exists in propDefinition
        if (propDefinition.hasOwnProperty("" + propValue)) {
            return parseDefinition(true, propDefinition[("" + propValue)], prefix);
        }
        // prop: {pseudo: value}
        return Object.keys(propDefinition).flatMap(key => {
            return parseDefinition(propValue, propDefinition[key], prefix + `${key}:`);
        });
    }
    return [];
}

const shouldMix = (when, optionArr) => {
    return when.every(w => {
        if (typeof w !== "string") {
            return false;
        }
        if (w.endsWith("*")) { // allow wildcards
            return optionArr.some(o => o.split(".").shift() === w.split(".").shift());
        }
        return optionArr.some(o => o === w); // simnple find
    });
};
const mixFunction = (when, run) => ({ when, run });
const mixAddClass = (when, arg) => {
    return {
        when,
        run: (css) => {
            const argCss = parseDefinition(true, arg);
            argCss.forEach(s => css.add(s));
        }
    };
};
const mixRemoveClass = (when, arg) => {
    return {
        when,
        run: (css) => {
            const argCss = parseDefinition(true, arg);
            argCss.forEach(s => css.delete(s));
        }
    };
};

/**
 * Hook that parses a set of parameters based on a configuration object and returns applicable classnames
 *
 *
 * @export
 * @template P extends ComponentAttributes
 * @template EL extends ExtendableElement = "div"
 * @param {ComposerConfig<P, EL>} config the configuration object
 * @param {CombinedProps<P, EL>} props the props to parse
 * @returns {ClassComposerReturns<A>} { className, forwardProps }
 */
function useClassComposer(config, props) {
    return useMemo(() => {
        // useMemo does nothing here, this should be an external hook with memoized return values
        const propOptions = [];
        const propCss = new Set();
        const optionEntries = Object.entries(config.options);
        const optionAlias = Object.entries(config.alias || []);
        const forwardProps = {};
        // load base
        parseDefinition(true, config.base).forEach(s => propCss.add(s));
        // load props
        for (const [key, propValue] of Object.entries(props)) {
            const entry = optionEntries
                .find(([k]) => {
                return k === key
                    || k === "$" + key
                    || k === "$$" + key
                    || optionAlias.find(([alias, target]) => alias === key && target === k);
            }); // find key or alias
            if (entry) {
                const [entryKey, entryValue] = entry;
                if (propValue && entryValue) {
                    parseDefinition(entryKey.startsWith("$$") ? true : propValue, entryValue).forEach(s => propCss.add(s));
                    propOptions.push(`${key}.${propValue}`);
                }
                if (entryKey.startsWith("$")) { // forward props starting with $
                    Object.assign(forwardProps, { [key]: propValue });
                }
            }
            else {
                // save prop to forward
                propOptions.push(`${key}.${propValue}`);
                Object.assign(forwardProps, { [key]: propValue });
            }
        }
        for (const mixer of config.mix || []) {
            if (shouldMix(mixer.when, propOptions)) {
                mixer.run(propCss);
            }
        }
        if (props.className) {
            props.className.split(" ").forEach((s) => propCss.add(s));
        }
        // merge with overwrite classname
        const className = Array.from(propCss).join(" ");
        return {
            forwardProps,
            className,
        };
    }, [props]);
}

/**
 * Create a React component that extends and creates a forwardRef of native HTML element.
 *
 *
 * @template P extends CustomAttributes
 * @template EL extends ExtendableElement
 * @param {EL} el defaults to "div"
 * @param {ComposerConfig<P, EL>} config
 * @param {Partial<P>} [defaults={}]
 * @returns {ComposedComponent<P, EL>}
 *
 * @see {ComposerConfig}
 * @see {ClassDefinition}
 */
function createComponent(el, config, defaults = {}) {
    return forwardRef((innerProps, ref) => {
        const ele = innerProps.as || el;
        const props = Object.assign(Object.assign({}, defaults), innerProps);
        delete props.as;
        const { className, forwardProps } = useClassComposer(config, props);
        return createElement(ele, Object.assign(Object.assign({ ref }, forwardProps), { className }), innerProps.children);
    });
}

function useClassname(definitions, deps = []) {
    return useMemo(() => {
        const css = new Set();
        parseDefinition(true, definitions).forEach(s => css.add(s));
        return Array.from(css).join(" ");
    }, [definitions, ...deps]);
}

export { createComponent, mixAddClass, mixFunction, mixRemoveClass, parseDefinition, shouldMix, useClassComposer, useClassname };
//# sourceMappingURL=index.js.map
