import { ComponentProps, PropsWithChildren, ForwardRefExoticComponent, PropsWithoutRef, RefAttributes, DependencyList } from 'react';

declare type AsProp = {
    as?: keyof JSX.IntrinsicElements;
};
declare type ExtendableElement = keyof JSX.IntrinsicElements | React.ElementType | React.ComponentType<any>;
declare type ComponentAttributes = Record<any, any>;
declare type CustomAttributes = {
    [k: string]: CustomAttributeValue | undefined;
};
declare type CustomAttributeValue = string | number | boolean;
/**
 * A Special type that does not allow on @S any key from @T
 *
 * @export
 * @typedef {Exclude}
 * @template S
 * @template T
 */
declare type Exclude<S, T> = S & {
    [K in keyof T]?: never;
};
/**
 * A class definition can be any of the following:
 * - A string with a single word: 'btn'
 * - A multiple word string: 'btn btn-primary'
 * - An array with more ClassDefinition[]
 * - A {key: ClassDefinition} object, where `key` will be used as a prefix
 * - A function that returns a ClassDefinition: () => ['a','b']
 *
 * Any ClassDefinition will be recursively parsed until it is just a string.
 *
 * @example: {
 *   a: 'btn',
 *   b: 'btn btn-primary',
 *   c: ['a', 'b', 'c'],
 *   d: ['a', { hover: 'b' }, ['c', 'd'], () => ({ dark: ['x', 'y', 'z'] })]
 * }
 *
 * @export
 * @typedef {ClassDefinition}
 */
declare type ClassDefinition = string | ClassDefinitionFn | {
    [key: string]: ClassDefinition;
} | ClassDefinition[];
declare type ClassDefinitionFn = ((value: CustomAttributeValue) => ClassDefinition);
declare type ComposerConfigOptions<T> = {
    [K in T as string]: ClassDefinition;
};
declare type OptionAlias<T> = {
    [key: string]: keyof T;
};
declare type ClassMixerFactory<T> = (when: string[], arg: T) => ClassMixer;
declare type ClassMixerFn = (css: Set<string>) => void;
declare type ClassMixer = {
    when: string[];
    run: ClassMixerFn;
};
interface ComposerConfig<P extends ComponentAttributes, EL extends ExtendableElement = "div"> {
    /**
     * base classes, will always be applied
     *
     * @type {ClassDefinition}
     * @see {ClassDefinition}
     */
    base: ClassDefinition;
    /**
     * options are `[key,value]` pairs, where the key is the prop name.
     *
     * - `key` should be present in the native HTML element props
     * - to use a native prop, prefix it with `$`
     * - to check if a prop is present, prefix it with `$$`
     *
     * @type {ExcludeFromHTMLElement<ComposerConfigOptions<P>, ComponentProps<EL> & AsProp>}
     */
    options: Exclude<ComposerConfigOptions<P>, ComponentProps<EL> & AsProp>;
    /**
     * A mixer is an object that looks like {when: [], run: () => {}}
     * and runs the function when the props match what was defined in the in the array
     *
     * Conditions can be specific like when: ['disabled.true'] or generic like ['title.*']
     *
     * @type {?ClassMixer[]}
     */
    mix?: ClassMixer[];
    /**
     * a simple {key: value} object, where value must be a key of {options}
     *
     * @type {?OptionAlias<P>}
     */
    alias?: OptionAlias<P>;
}
/**
 * Returns the classnames parsed from the options object and a set of properties to forward
 *
 * @export
 * @interface ClassComposerReturns
 * @typedef {ClassComposerReturns}
 * @template E extends ExtendableElement
 */
interface ClassComposerReturns<EL extends ExtendableElement> {
    /**
     * a string with css classnames
     */
    className: string;
    /**
     * Contains any prop that is native to the HTML Element being forwarded
     *
     * @type {PropsWithChildren<ComponentProps<EL>>}
     */
    forwardProps: PropsWithChildren<ComponentProps<EL>>;
}
/**
 * A component that was created using createComponent, will be an instance of EL and  allow any property present in P and A
 *
 * @export
 * @typedef {ComposedComponent}
 * @template P extends CustomAttributes
 * @template EL extends ExtendableElement
 */
declare type ComposedComponent<P extends CustomAttributes, EL extends ExtendableElement> = ForwardRefExoticComponent<PropsWithoutRef<ComposedComponentProps<P, EL>> & RefAttributes<EL>>;
/**
 * Combines Props P with Component Pros from element EL and the {as?: ""} prop
 *
 * @export
 * @typedef {ComposedComponentProps}
 * @template P extends CustomAttributes
 * @template EL extends ExtendableElement
 */
declare type ComposedComponentProps<P extends CustomAttributes, EL extends ExtendableElement> = P & AsProp & ComponentProps<EL>;

/**
 * Create a React component that extends and creates a forwardRef of native HTML element.
 *
 *
 * @template P extends CustomAttributes
 * @template EL extends ExtendableElement
 * @param {EL} el defaults to "div"
 * @param {ComposerConfig<P, EL>} config
 * @param {Partial<P>} [defaults={}]
 * @returns {ComposedComponent<P, EL>}
 *
 * @see {ComposerConfig}
 * @see {ClassDefinition}
 */
declare function createComponent<P extends CustomAttributes, EL extends ExtendableElement = "div">(el: EL, config: ComposerConfig<P, EL>, defaults?: Partial<P>): ComposedComponent<P, EL>;

declare function parseDefinition(propValue: CustomAttributeValue, propDefinition: ClassDefinition, prefix?: string): string[];

declare const shouldMix: (when: string[], optionArr: string[]) => boolean;
declare const mixFunction: ClassMixerFactory<ClassMixerFn>;
declare const mixAddClass: ClassMixerFactory<ClassDefinition>;
declare const mixRemoveClass: ClassMixerFactory<ClassDefinition>;

/**
 * Hook that parses a set of parameters based on a configuration object and returns applicable classnames
 *
 *
 * @export
 * @template P extends ComponentAttributes
 * @template EL extends ExtendableElement = "div"
 * @param {ComposerConfig<P, EL>} config the configuration object
 * @param {CombinedProps<P, EL>} props the props to parse
 * @returns {ClassComposerReturns<A>} { className, forwardProps }
 */
declare function useClassComposer<P extends ComponentAttributes, EL extends ExtendableElement = "div">(config: ComposerConfig<P, EL>, props: ComposedComponentProps<P, EL>): ClassComposerReturns<EL>;

declare function useClassname(definitions: ClassDefinition, deps?: DependencyList[]): string;

export { AsProp, ClassComposerReturns, ClassDefinition, ClassDefinitionFn, ClassMixer, ClassMixerFactory, ClassMixerFn, ComponentAttributes, ComposedComponent, ComposedComponentProps, ComposerConfig, ComposerConfigOptions, CustomAttributeValue, CustomAttributes, Exclude, ExtendableElement, OptionAlias, createComponent, mixAddClass, mixFunction, mixRemoveClass, parseDefinition, shouldMix, useClassComposer, useClassname };
